/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.platform.raspberry.generator.extension.controlcenter;

import org.fortiss.af3.component.model.InputPort;
import org.fortiss.af3.expression.model.types.TBool;
import org.fortiss.af3.expression.model.types.TDouble;
import org.fortiss.af3.expression.model.types.TInt;
import org.fortiss.af3.platform.raspberry.generator.framework.IReadableGeneratorExtension;
import org.fortiss.af3.platform.raspberry.model.controlcenter.ControlCenterDownstreamInput;

/** Executable for controlcenter {@link ControlCenterDownstreamInput}. */
public class ControlCenterDownstreamInputGeneratorExtension extends
		ControlcenterGeneratorExtensionBase<ControlCenterDownstreamInput> implements
		IReadableGeneratorExtension {
	/** Constructor. */
	public ControlCenterDownstreamInputGeneratorExtension(ControlCenterDownstreamInput modelElement) {
		super(modelElement);
	}

	/** {@inheritDoc} */
	@Override
	public String getReadCode(InputPort logicalSignal, String targetVariable) {
		// TODO add NoVal Parameter to getVariable
		String portInputName = modelElement.getName();
		// get the value depending on the logical signal data type
		if(logicalSignal.getVariableType() instanceof TDouble) {
			return "af3_cc_get_double(\"" + modelElement.getName() + "\" , " + "&" +
					targetVariable + ", &noval_" + targetVariable + " );\n";
		} else if(logicalSignal.getVariableType() instanceof TInt) {
			return "af3_cc_get_int(\"" + modelElement.getName() + "\" , " + "&" + targetVariable +
					", &noval_" + targetVariable + " );\n";
		} else if(logicalSignal.getVariableType() instanceof TBool) {
			return "af3_cc_get_bool(\"" + modelElement.getName() + "\" , " + "&" + targetVariable +
					", &noval_" + targetVariable + " );\n";
		} else {
			return "FIXME(\"Data type " + logicalSignal.getVariableType() + " of Port " +
					portInputName + " not supported for control centert.\");\n";
		}

	}

	/** {@inheritDoc} */
	@Override
	public String getNoValReadCode(InputPort logicalSignal) {
		// TODO Auto-generated method stub
		return null;
	}
}
